#include "ObjetAppelPremiers.h"
#include <limits>
#include "OutilsServeurPremiers.h"


unsigned long __stdcall ObjetAppelPremiers::AddRef_NoAggregation()
{
   return ++m_CptRef;
}
unsigned long __stdcall ObjetAppelPremiers::Release_NoAggregation()
{
   if (--m_CptRef == 0)
   {
      delete this;
      return 0;
   }
   return m_CptRef;
}

HRESULT __stdcall ObjetAppelPremiers::QueryInterface_NoAggregation(REFIID riid, void** ppv)
{
	if(riid == IID_IUnknown)
   {
		*ppv = static_cast<INoAggregationUnknown*>(this); // hum
   }
	else if(riid == IID_AsyncIPremier)
   {
		*ppv = static_cast<AsyncIPremier*>(this);
   }
	else 
	{
		*ppv = NULL;
		return E_NOINTERFACE;
	}
	static_cast<IUnknown*>(*ppv)->AddRef();
	return S_OK;
}

// ----------------------------------------------------
// Implmentation des mthodes de la classe ObjetAppelPremiers
// (pour AsyncIPremier) (version nave)
// ----------------------------------------------------


HRESULT __stdcall ObjetAppelPremiers::Begin_EstPremier( int Nb )
{
   int EnTest = static_cast<int>(sqrt (Nb));

   while (EnTest >= 2) // plus petit diviseur intressant
   {
      ICancelMethodCalls* pCancelMethodCalls = 0;
      CoGetCallContext(IID_ICancelMethodCalls, reinterpret_cast<void**>(&pCancelMethodCalls));
      HRESULT hr = pCancelMethodCalls->TestCancel();
      pCancelMethodCalls->Release();

      // L'appel a-t-il t annul?
      if(hr == RPC_E_CALL_CANCELED)
         return RPC_E_CALL_CANCELED;


      if (Nb % EnTest == 0) // le nombre est divisible par EnTest
      {
         m_ResultatEstPremier = false; // pas premier
         m_pSync->Signal (); // fini!
         return S_OK;
      }
      EnTest --;
   }

	m_ResultatEstPremier = true;
	m_pSync->Signal();
	return S_OK;
}

HRESULT __stdcall ObjetAppelPremiers::Finish_EstPremier( boolean *retval)
{
   m_pSync->Wait (0, INFINITE);
   *retval = m_ResultatEstPremier? TRUE : FALSE;
   return S_OK;
}


#undef max /* entrait en conflit avec std::numeric_limits<int>::max() */

HRESULT __stdcall ObjetAppelPremiers::Begin_NiemePremier( int Lequel )
{
   int NombreTeste = 2; // plus petit premier
   int NbPremiers = 0;
   while (NbPremiers < Lequel && NombreTeste < std::numeric_limits<int>::max())
   {
      ICancelMethodCalls* pCancelMethodCalls = 0;
      CoGetCallContext(IID_ICancelMethodCalls, reinterpret_cast<void**>(&pCancelMethodCalls));
      HRESULT hr = pCancelMethodCalls->TestCancel();
      pCancelMethodCalls->Release();

      // L'appel a-t-il t annul?
      if(hr == RPC_E_CALL_CANCELED)
         return RPC_E_CALL_CANCELED;

      if (EntierEstPremier (NombreTeste))
      {
         NbPremiers ++;
         if (NbPremiers == Lequel)
         {
            m_ResultatNiemePremier = NombreTeste;
            m_pSync->Signal ();
            return S_OK;
         }
      }
      NombreTeste ++;
   }
   m_ResultatNiemePremier = -1;
   m_pSync->Signal ();
   return S_OK;


}

HRESULT __stdcall ObjetAppelPremiers::Finish_NiemePremier( int *retval)
{
   m_pSync->Wait (0, INFINITE);
   *retval = m_ResultatNiemePremier;
   return S_OK;
}

