#include <iostream>
#include <string>
#include <string.h>

#define _WIN32_DCOM
#include <windows.h>

#include "../Commun/Fabriques--ServeurPremiers.h"
#include "../../StdOutilsCOM_Lib/Registry.h"

#include "../../StdOutilsCOM_Lib/GestionnaireDeVerrou.h"
#include "FabriqueServeurPremiers.h"


const std::string ENTETE_VALIDE_DOS  = "/",
                  ENTETE_VALIDE_UNIX = "-";


inline bool ComparerChaineCaseInsensitive (const std::string &s1,
                                           const std::string &s2)
{
   return stricmp (s1.c_str (), s2.c_str ()) == 0;
}


inline bool EnTeteParamValide (const std::string &Param)
{
   return Param.substr (0, ENTETE_VALIDE_DOS.length ())  == ENTETE_VALIDE_DOS ||
          Param.substr (0, ENTETE_VALIDE_UNIX.length ()) == ENTETE_VALIDE_UNIX;
}

inline std::string ExtraireDetailParam (const std::string &Param)
{
   std::string Resultat;
   if (Param.substr (0, ENTETE_VALIDE_DOS.length ())  == ENTETE_VALIDE_DOS)
   {
      Resultat = Param.substr (ENTETE_VALIDE_DOS.length ());
   }
   else if (Param.substr (0, ENTETE_VALIDE_UNIX.length ())  == ENTETE_VALIDE_UNIX)
   {
      Resultat = Param.substr (ENTETE_VALIDE_UNIX.length ());
   }
   return Resultat;
}

inline bool EstInscription (const std::string &Param)
{
   const std::string COMTOK_INSCRIPTION = "RegServer";
   return ComparerChaineCaseInsensitive (Param, COMTOK_INSCRIPTION);
}

inline bool EstDesinscription (const std::string &Param)
{
   const std::string COMTOK_DESINSCRIPTION = "UnregServer";
   return ComparerChaineCaseInsensitive (Param, COMTOK_DESINSCRIPTION);
}

inline bool EstLancementViaCOM (const std::string &Param)
{
   // Le mot cl Embedding remonte aux racines de COM, qui est un
   // driv du standard (interne) OLE
   const std::string COMTOK_LANCEMENT_VIA_COM = "Embedding";
   return ComparerChaineCaseInsensitive (Param, COMTOK_LANCEMENT_VIA_COM);
}

void LancerServeur ();
void TuerServeur ();

int main (int argc, char *argv [])
{
   const int SUCCES = 0;

   if (argc > 1)
   {
      const std::string PARAM = argv[1];

      if (EnTeteParamValide (PARAM))
      {
         const std::string DETAIL_PARAM = ExtraireDetailParam (PARAM);
         if (EstInscription (DETAIL_PARAM))
         {
            // La fonction RegisterServer () examine si le nom
            // du serveur se termine par .dll ou par .exe, et
            // inscrit le composant dans la bonne cl du registre
            // (InprocServer32 pour un .dll, LocalServer32 pour
            // un .exe)
            HRESULT hr;
            std::cout << "Enregistrement du serveur " << argv[0] << "...";
            hr = RegisterServer (argv[0],
                                 CLSID_FabriqueAnalystePremiers,
                                 "Fabrique de serveurs de premiers",
                                 "Component.OutOfProc.SrvPremiers",
                                 "Component.OutOfProc.SrvPremiers.1",
                                 0 ); // NULL
            if (SUCCEEDED (hr))
            {
               std::cout << " russi!" << std::endl;
            }
            else
            {
               std::cout << " chou, code " << hr << std::endl;
            }
            return hr;
         }
         else if (EstDesinscription (DETAIL_PARAM))
         {
            HRESULT hr;
            std::cout << "Dsinscription du serveur " << argv[0] << "...";
            hr = UnregisterServer (CLSID_FabriqueAnalystePremiers,
                                   "Component.OutOfProc.SrvPremiers",
                                   "Component.OutOfProc.SrvPremiers.1");
            if (SUCCEEDED (hr))
            {
               std::cout << " russi!" << std::endl;
            }
            else
            {
               std::cout << " chou, code " << hr << std::endl;
            }
            return hr;
         }
         else if (EstLancementViaCOM (DETAIL_PARAM))
         {
            std::cout << "-----------------------------" << std::endl
                      << "Serveur de premiers..." << std::endl
                      << "Fait par votre chic prof..." << std::endl
                      << "-----------------------------" << std::endl;
            LancerServeur ();
            GestionnaireDeVerrou::AttendreFinDuServeur ();
            TuerServeur ();
         }
         else
         {
            std::cerr << "Serveur "
                      << argv[0]
                      << ", parametre inconnu: "
                      << argv[1]
                      << std::endl;
         }
      }  
   }
   else
   {
      std::cerr << "Tentative de lancement de "
                << argv[0]
                << " en tant que processus autonome..."
                << std::endl;
   }
   return SUCCES;
}




// ----------------------------------------------------
// ----------------------------------------------------
// Dtails propres au serveur COM out-of process
// ----------------------------------------------------
// ----------------------------------------------------

DWORD EnregistrementFabrique;
FabriqueServeurPremiers *pFab;

void LancerServeur ()
{
   CoInitializeEx (0, COINIT_MULTITHREADED);
   CoInitializeSecurity( 0, // descripteur de scurit nul (pas de descripteur)
                        -1, // nombre d'lments dans le tableau donn (paramtre suivant)
                         0, // tableau de services d'authentification supports (ici: nul)
                         0, // rserv
                         RPC_C_AUTHN_LEVEL_NONE, // tant pis pour l'authentification
                         RPC_C_IMP_LEVEL_IDENTIFY, // tant pis pour la personnification
                         0, // rserv
                         EOAC_NONE , // capacits d'authentification (aucune)
                         0 ); // rserv
   pFab = new FabriqueServeurPremiers ();
   ::CoRegisterClassObject (CLSID_FabriqueAnalystePremiers,
                            static_cast<IClassFactory *>(pFab),
                            CLSCTX_SERVER,
                            REGCLS_MULTIPLEUSE | REGCLS_SUSPENDED,
                            &EnregistrementFabrique);
   CoResumeClassObjects ();
}

void TuerServeur ()
{
   CoRevokeClassObject (EnregistrementFabrique);
   pFab -> Release ();
   CoUninitialize ();
}
